rawset(_G, "Bananaporter", {})

local B = Bananaporter

//-------------------------------------------//

freeslot("MT_BANANAPORTER", "SPR_LITE")

mobjinfo[MT_BANANAPORTER] = {
	doomednum = 5000,
	spawnstate = S_INVISIBLE,
	flags = MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT
}

//-------------------------------------------//

local function activateBananaporter(mo)
	S_StartSound(mo, sfx_3db06)

	local sec = mo.subsector.sector

	mo.sides = {}

	for i = 0, #sec.lines - 1
		local line = sec.lines[i]
		
		local x = (line.v1.x + line.v2.x)/2
		local y = (line.v1.y + line.v2.y)/2
		local z = sec.floorheight
		local ang = R_PointToAngle2(line.v1.x, line.v1.y, line.v2.x, line.v2.y)
		local side = P_SpawnMobj(x, y, z - 4*FRACUNIT, MT_THOK)
		
		side.angle = ang
		side.spritexscale = R_PointToDist2(line.v1.x, line.v1.y, line.v2.x, line.v2.y)/12 -- sprite width
		side.tics = -1
		side.sprite = SPR_LITE
		side.renderflags = $ | RF_PAPERSPRITE
		--side.blendmode = AST_ADD
		side.frame = $ | FF_TRANS70
		side.color = mo.portalcolor
		side.colorized = true
		
		table.insert(mo.sides, side)
	end
	
	mo.active = true
end

addHook("MapThingSpawn", function(mo, mt)
	mo.portaltag = mt.tag
	
	B[gamemap] = $ or {}
	B[gamemap].portalmobjs = $ or {}
	B[gamemap].portalmobjs[mt.tag] = $ or {}
	
	if mt.args[0]
		B[gamemap].portalmobjs[mt.tag][2] = mo
		
		mo.triggerer = true -- this portal will activate the other
		mo.location = mt.stringargs[0] -- activating this portal will have a location flavor text
	else
		B[gamemap].portalmobjs[mt.tag][1] = mo
	end
	
	mo.pair = B[gamemap].portalmobjs[mt.tag]
	mo.portalcolor = R_GetColorByName(tostring(mt.stringargs[1]))
end, MT_BANANAPORTER)

addHook("MapLoad", function()
	if not B return end
	if not B[gamemap] return end
	
	for _, tag in pairs(B[gamemap].portalmobjs)
		if not tag[1] or not tag[2] continue end
		
		tag[1].link = tag[2]
		tag[2].link = tag[1]
	end
	
	// Luabanks functionality for this will only work for the hub because I don't have time for this
	if not OLDC or not OLDCsave or gamemap != OLDC.hub return end
	
	for index, tag in pairs(B[gamemap].portalmobjs)
		if (OLDCsave[3] & (1 << (index - 1000)))
			tag.active = true
		end
	end
end)

addHook("GameQuit", do
	if multiplayer return end
	if not OLDC or not OLDCsave return end
	if not B[OLDC.hub] or B[OLDC.hub].portalmobjs return end -- ?!
	
	for index, tag in pairs(B[OLDC.hub].portalmobjs)
		if tag.active
			OLDCsave[3] = $ | (1 << (index - 1000))	
		end
	end
end)

local displayticker = 0
local displaylocation

addHook("PlayerThink", function(p)
	if not B return end
	if not B[gamemap] return end
	
	p.bananaporter = nil
	p.canbananaport = false
	p.bananaporter_cd = $ or 0
	
	if p.bananaporter_cd and not (p.cmd.buttons & BT_SPIN)
		p.bananaporter_cd = max(0, $ - 1)
	end

	if not p.mo return end
	if not p.mo.valid return end
	if p.playerstate return end
	if not P_IsObjectOnGround(p.mo) return end
	
	local sec = p.mo.subsector.sector
	
	for mobj in sec.thinglist()
		if mobj.type != MT_BANANAPORTER continue end
		if not mobj.portaltag continue end
		if mobj.floorz != p.mo.floorz continue end
		
		p.bananaporter = mobj
	end
	
	if p.bananaporter
		if not p.bananaporter.pair.active
			if p.bananaporter.triggerer
				p.bananaporter.pair.active = true
				
				if (OLDC and gamemap == OLDC.hub) and OLDCsave
					OLDCsave[3] = $ | (1 << (p.bananaporter.portaltag - 1000))
				end
				
				displaylocation = p.bananaporter.location
				displayticker = 3*TICRATE
			else return end
		end

		if p.speed return end
		if p.powers[pw_carry] return end
		if p.bananaporter_cd return end
		if OLDC and OLDC.Data.voteInfo.voting return end
		
		p.canbananaport = true
	end
end)

addHook("SpinSpecial", function(p)
	if not p.bananaporter return end
	if not p.canbananaport return end
	
	P_SetOrigin(p.mo, p.bananaporter.link.x, p.bananaporter.link.y, p.bananaporter.link.z)
				
	p.mo.angle = p.bananaporter.link.angle
	p.drawangle = p.mo.angle
	
	p.bananaporter_cd = TICRATE
	
	S_StartSound(p.mo, sfx_s3kc3s, p)
	
	return true
end)

addHook("MobjThinker", function(mo)
	if leveltime < 3 return end
	if not mo.pair return end
	if not mo.pair.active return end
	
	if not mo.sides
		activateBananaporter(mo)
	end
	
	for _, side in ipairs(mo.sides)
		side.spriteyscale = (sin(FixedAngle(3*leveltime*FRACUNIT))/2 + FRACUNIT/2)/8 + FRACUNIT/6
	end
end, MT_BANANAPORTER)

addHook("HUD", function(v, p)
	if displayplayer != consoleplayer return end
	
	if p.bananaporter
	and p.bananaporter.pair
	and p.bananaporter.pair.active
		local x = 150*FRACUNIT
		local y = 180*FRACUNIT
		local button = v.cachePatch("BUTTON"..((leveltime/TICRATE) % 2) + 3)
		local offs = 5*FRACUNIT
		local cd = FixedDiv(TICRATE - p.bananaporter_cd, TICRATE)
		
		v.drawScaled(x - offs/2, y + 3*FRACUNIT, FRACUNIT/2, button, V_TRANSLUCENT)
		v.drawCropped(x - offs/2, y + 3*FRACUNIT, FRACUNIT/2, FRACUNIT/2, button, 0, nil, 0, 0, button.width*cd, button.height*FRACUNIT)
		v.drawString(x + offs, y, "Warp", (p.bananaporter_cd and V_TRANSLUCENT or 0)|V_ALLOWLOWERCASE, "thin-fixed")
	end

	if not displayticker return end
	
	local fadein = min(3*TICRATE - displayticker, 9)
	local fadeout = min(displayticker, 9)
	local valuetouse = 9
	
	if displayticker < TICRATE then valuetouse = fadeout end
	if displayticker > TICRATE*2 then valuetouse = fadein end
	
	v.drawStretched(0, 105*FRACUNIT - 5*FixedDiv(valuetouse, 9), v.width()*FRACUNIT/64, 10*FixedDiv(valuetouse, 9)/64, v.cachePatch("~031"), V_70TRANS|V_SNAPTOLEFT)
	v.drawString(160, 102, "Fast travel to "..displaylocation.." has been unlocked.", ((9 - valuetouse) << V_ALPHASHIFT)|V_ALLOWLOWERCASE, "thin-center")
	
	displayticker = $ and $ - 1 or 0
end)

addHook("NetVars", function(net)
	Bananaporter = net($)
end)