
freeslot("MT_FOOD",
		 "SPR_FOO1", "SPR_FOO2", "SPR_FOO3", "SPR_FOO4", "SPR_FOO5",
		 "SPR_FOO6", "SPR_FOO7", "SPR_FOO8", "SPR_FOO9", "SPR_FOOA",
		 "SPR_FOOB", "SPR_FOOC", "SPR_FOOD", "SPR_FOOE", "SPR_FOOF",
		 "SPR_FOOG", "SPR_FBIT",
		 "S_FOOD",
		 "sfx_cru1", "sfx_cru2", "sfx_cru3",
		 "sfx_slp1", "sfx_slp2", "sfx_slp3",
		 "sfx_pidin")

local cronch = {sfx_cru1, sfx_cru2, sfx_cru3}
local sipp 	 = {sfx_slp1, sfx_slp2, sfx_slp3}
local pizzatime = 0

// Food sprites
rawset(_G, "foods", 
	{SPR_FOO1, SPR_FOO2, SPR_FOO3, SPR_FOO4, SPR_FOO5, SPR_FOO6, SPR_FOO7, SPR_FOO8, SPR_FOO9, SPR_FOOA,
     SPR_FOOB, SPR_FOOC, SPR_FOOD, SPR_FOOE, SPR_FOOF, SPR_FOOG}
)

// Which of the foods above are considered drinks
rawset(_G, "drinks",
	{SPR_FOOB, SPR_FOOC, SPR_FOOD, SPR_FOOE, SPR_FOOF, SPR_FOOG}
)

// SOC and ZB stuff
mobjinfo[MT_FOOD] = {
	doomednum = 4020,
	spawnstate = S_FOOD,
	radius = 16*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_SPECIAL,
	--$Name Food
	--$Sprite FOO1A0
	--$AngleText Food sprite
	--$Flags1Text Respawning
	--$Flags4Text 2X Rings
	--$Flags8Text 4X Rings
	--$ParameterText Rings
}

// Our one and only state
states[S_FOOD] = {
	sprite = SPR_BURG,
	frame = A,
	tics = -1,
	nextstate = S_FOOD
}

// You eat like a fooken pig good lord
sfxinfo[sfx_cru1].caption = "Crunch"
sfxinfo[sfx_cru2].caption = "Crunch"
sfxinfo[sfx_cru3].caption = "Crunch"

sfxinfo[sfx_slp1].caption = "Slurp"
sfxinfo[sfx_slp2].caption = "Slurp"
sfxinfo[sfx_slp3].caption = "Slurp"

sfxinfo[sfx_pidin].caption = "Pizza time"

// Stuff to do when the food spawns. Give it a shadow, its proper sprite and ring recovery value
addHook("MapThingSpawn", function(mo, thing)
	if not mo.valid return end
	mo.scale = 1
	mo.destscale = FRACUNIT
	mo.shadowscale = FRACUNIT - FRACUNIT/4

	local food = thing.angle
	if food >= #foods then food = #foods - 1 end
	mo.sprite = foods[food + 1]
	mo.recovery = thing.extrainfo
	if (thing.options & MTF_OBJECTSPECIAL) then mo.recovery = $ * 2 end
	if (thing.options & MTF_AMBUSH) then mo.recovery = $ * 4 end
	for i, drink in ipairs(drinks) if mo.sprite == drink then mo.drink = true end end
	if (thing.options & MTF_EXTRA) then mo.respawning = true end
end, MT_FOOD)

// The food was touched. Spawn some bits of food, play a sound, recover rings
// If the food is respawnable, start its respawn timer
addHook("TouchSpecial", function(mo, pmo)
	if not pmo or not pmo.valid return end
	if not pmo.player or not pmo.player.valid return end

	// Give our rings, play the eating sounds
	P_GivePlayerRings(pmo.player, mo.recovery)
	S_StartSound(pmo, mo.drink and sipp[P_RandomRange(1, #sipp)] or cronch[P_RandomRange(1, #cronch)] )

	// Spawn the bits
	for i=1, P_RandomRange(2, 6)
		local foodbit = P_SpawnMobjFromMobj(pmo, 0, 0, pmo.height, MT_UNKNOWN)
		foodbit.flags = $ & ~MF_NOGRAVITY
		foodbit.sprite = SPR_FBIT
		foodbit.frame = mo.drink and $ + 3 or $ + P_RandomRange(0, 2)
		foodbit.scale = FRACUNIT + P_RandomRange(0, FRACUNIT-500)
		foodbit.rollangle = P_RandomRange(-45, 45) * ANG1
		foodbit.momx = P_RandomRange(-5, 5) * FRACUNIT
		foodbit.momy = P_RandomRange(-5, 5) * FRACUNIT
		P_SetObjectMomZ(foodbit, P_RandomRange(2, 4) * FRACUNIT)
	end

	if not mo.spawnpoint return end

	// Should the food respawn? If so, set some stuff here so it appears again later
	if mo.respawning
		mo.flags = $ & ~MF_SPECIAL
		mo.flags2 = $|MF2_DONTDRAW
		mo.fuse = 8*TICRATE
		mo.scale = 1
		return true
	end
end, MT_FOOD)

// Respawn our food, giving it all its proper flags back
addHook("MobjFuse", function(mo)
	if not mo.valid return end
	if not mo.respawning return end

	mo.flags = $|MF_SPECIAL
	mo.flags2 = $ & ~MF2_DONTDRAW
	mo.destscale = FRACUNIT
	return true
end, MT_FOOD)

// Behold! The return of orderpizza!
COM_AddCommand("orderpizza", function(p, arg)
	if not p.valid return end
	if not p.mo or not p.mo.valid return end
	if pizzatime
		CONS_Printf(p, "\x04It's still too soon to order another pizza. Give the cook some time to breathe!")
		return
	end
	print("\x82" + p.name + " has ordered some pizza for everyone!")
	for i=1, 8
		local pizza = P_SpawnMobjFromMobj(p.mo, 0, 0, 0, MT_FOOD)
		pizza.scale = 1
		pizza.destscale = FRACUNIT
		pizza.shadowscale = FRACUNIT - FRACUNIT/4
		pizza.sprite = SPR_FOO5
		pizza.recovery = 0
		pizza.fuse = 30 * TICRATE
		pizza.angle = (i-1) * 45 * ANG1
		P_TeleportMove(pizza, 
					   p.mo.x + 120 * cos(p.mo.angle),
					   p.mo.y + 120 * sin(p.mo.angle),
					   p.mo.z)
		P_TeleportMove(pizza, pizza.x + 40 * cos(pizza.angle), pizza.y + 40 * sin(pizza.angle), pizza.z)
	end
	S_StartSound(nil, sfx_pidin)
	pizzatime = 30 * TICRATE
end)

// Sync our pizza time when online
addHook("NetVars", function(net)
	pizzatime = net(pizzatime)
end)

// Decrease the pizza time wait every tic
addHook("ThinkFrame", function()
	if pizzatime > 0 then pizzatime = $ - 1 end
end)