freeslot("MT_OLDC_INFOSCREEN", "sfx_hbinfo")

mobjinfo[MT_OLDC_INFOSCREEN] = {
	doomednum = 5001,
	spawnstate = S_INVISIBLE,
	radius = 32*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_SPECIAL
}

addHook("MapThingSpawn", function(mo, mt)
	local target = P_SpawnMobj(mo.x, mo.y, mo.floorz, MT_THOK)
	target.tics = -1
	target.sprite = SPR_HTRG
	target.frame = A
	target.renderflags = $ | (RF_FLOORSPRITE|RF_OBJECTSLOPESPLAT|RF_NOSPLATBILLBOARD)
end, MT_OLDC_INFOSCREEN)

addHook("TouchSpecial", function(mo, toucher)
	if not toucher.player return true end
	if toucher.player.speed return true end
	if toucher.player.powers[pw_carry] return true end
	if not P_IsObjectOnGround(toucher) return true end
	if OLDC and OLDC.Data.voteInfo.voting return true end
	
	toucher.player.oldc_infoscreen = mo
	
	return true
end, MT_OLDC_INFOSCREEN)

addHook("MapChange", function(map)
	if not OLDC or not OLDCsave return end
	
	if gamemap == OLDC.hub
	and map - OLDC.firstmap < TOTALENTRIES
	and map - OLDC.firstmap >= 0
		OLDC.Data.visitedChecks[map] = true
		OLDCsave[2] = $ | (1 << (map - OLDC.firstmap))
	end
	
	if map == OLDC.hub
		for i = 0, TOTALENTRIES - 1
			if OLDCsave[2] & (1 << i)
				OLDC.Data.visitedChecks[i + OLDC.firstmap] = true
			end
		end
	end
end)

addHook("GameQuit", do
	if not OLDC or not OLDCsave return end
	
	for i = 0, TOTALENTRIES - 1
		if OLDC.Data.visitedChecks[i + OLDC.firstmap]
			OLDCsave[2] = $ | (1 << i)
		end
	end
end)

addHook("PlayerThink", function(p)
	if not p.mo or not p.mo.valid
		p.oldc_infoscreen = nil
		p.oldc_infoscreen_interacting = false
		return
	end
	
	if not p.oldc_infoscreen return end
	
	if not p.oldc_infoscreen.valid
		p.oldc_infoscreen = nil
		p.oldc_infoscreen_interacting = false
		return
	end
	
	local screen = p.oldc_infoscreen
		
	if R_PointToDist2(0, 0, R_PointToDist2(p.mo.x, p.mo.y, screen.x, screen.y), p.mo.z - screen.z) >= screen.radius
		p.oldc_infoscreen = nil
		p.oldc_infoscreen_interacting = false
		return
	end
end)

local sp = true
local buttoncd = 0
local displaytimer = 0

addHook("SpinSpecial", function(p)
	if not p.oldc_infoscreen return end
	if p.lastbuttons & BT_SPIN return end
	
	p.oldc_infoscreen_interacting = not $
	buttoncd = 3
	
	S_StartSound(nil, sfx_hbinfo, p)
	
	return true
end)

addHook("ThinkFrame", do buttoncd = max(0, $ - 1) end)

local topitem = 0
local maxitems = TOTALENTRIES - 13

addHook("PlayerCmd", function(p, cmd)
	if not p.oldc_infoscreen_interacting return end
		
	if cmd.buttons & BT_JUMP
		cmd.buttons = $ & ~BT_JUMP
		
		if not buttoncd
			sp = not $
			
			S_StartSound(nil, sfx_menu1, p)
		end
		
		buttoncd = 3
	end
	
	if cmd.forwardmove != 0 and TOTALENTRIES > 13
		if sp and not buttoncd
			if cmd.forwardmove > 0
				if topitem > 0
					topitem = $ - 1
					S_StartSound(nil, sfx_menu1, p)
				end
			else
				if topitem < maxitems
					topitem = $ + 1
					S_StartSound(nil, sfx_menu1, p)
				end
			end
		end
		
		buttoncd = 2
	end
	
	cmd.forwardmove = 0
	cmd.sidemove = 0
end)

local mplevelslist = {
	{"M0", "Match"},
	{"R0", "Circuit"},
	{"R1", "Circuit"},
	{"SA", "Circuit"},
}

local function GetMapName(map)
	local name = mapheaderinfo[map].lvlttl
		
	if not (mapheaderinfo[map].levelflags & LF_NOZONE)
		name = $.." Zone"
	end
		
	if mapheaderinfo[map].actnum
		name = $..", Act "..mapheaderinfo[map].actnum
	end
		
	return name
end

addHook("HUD", function(v, p)
	if displayplayer != consoleplayer return end
	
	if p.oldc_infoscreen and not p.oldc_infoscreen_interacting
		local x = 140*FRACUNIT
		local y = 180*FRACUNIT
		local button = v.cachePatch("BUTTON"..((leveltime/TICRATE) % 2) + 3)
		local offs = 5*FRACUNIT
		
		v.drawScaled(x - offs/2, y + 3*FRACUNIT, FRACUNIT/2, button)
		v.drawString(x + offs, y, "Interact", V_ALLOWLOWERCASE, "thin-fixed")
	end
	
	if not paused
		if p.oldc_infoscreen_interacting then displaytimer = min(16, $ + 1) end
		if not p.oldc_infoscreen_interacting then displaytimer = max(0, $ - 1) end
	end
	
	if OLDC and OLDC.Data.voteInfo.voting
		displaytimer = 0
	end
	
	if not OLDC or not displaytimer return end
	
	local bg = v.cachePatch("~134")
	local pivotx = 160*FRACUNIT
	local pivoty = 100*FRACUNIT
	local sizex = 240
	local sizey = 160
	
	local xscaletimer = min(8, displaytimer)
	local yscaletimer = min(8, max(0, displaytimer - 8))
	local xscale = FixedDiv(xscaletimer, 8)
	local yscale = max(FRACUNIT/64, FixedDiv(yscaletimer, 8))
	
	v.drawStretched(pivotx - xscale*sizex/2, pivoty - xscale*sizey/2, xscale*sizex/64, yscale*sizey/64, bg, V_TRANSLUCENT)
	
	if displaytimer < 16 return end
	
	local yoffs = 8*FRACUNIT
	local startx = pivotx - sizex*FRACUNIT/2 + 8*FRACUNIT
	local starty = pivoty - sizey*FRACUNIT/2 + 8*FRACUNIT
	local title = sp and "Single Player" or "Multiplayer"
	
	v.drawString(pivotx, starty, title, V_YELLOWMAP, "fixed-center")
	
	starty = $ + 6*FRACUNIT
	
	if sp
		local cleared = 0
		
		for i = 0, TOTALENTRIES - 1
			if OLDC.Data.mapChecks[i + OLDC.firstmap] then cleared = $ + 1 end
		end
		
		local offs = cleared > 9 and 4 or 0
		
		v.drawStretched(startx + (sizex - 50 - offs)*FRACUNIT, starty - 8*FRACUNIT, (38 + offs)*FRACUNIT/64, 12*FRACUNIT/64, bg)
		v.drawString(startx + (sizex - 28)*FRACUNIT, starty - 6*FRACUNIT, cleared.."/"..TOTALENTRIES, 0, "thin-fixed-right")
		v.drawScaled(startx + (sizex - 20)*FRACUNIT, starty - 2*FRACUNIT, FRACUNIT/3, v.cachePatch("MAPCLEAR"), 0)
	
		startx = $ + 54*FRACUNIT
		
		local arrowpos = ((leveltime/15) % 2) and FRACUNIT or 0
		
		if topitem > 0
			v.drawString(startx - 12*FRACUNIT, starty + 10*FRACUNIT + arrowpos, "\026", V_YELLOWMAP, "fixed")
		end
		
		for i = topitem, maxitems + topitem
			if starty + 10*FRACUNIT >= pivoty + sizey*FRACUNIT/2 - 8*FRACUNIT then break end
			
			starty = $ + 10*FRACUNIT
			
			--local j = (i % 4) + 1
			local clear = OLDC.Data.mapChecks[OLDC.firstmap + i]
			local visited = OLDC.Data.visitedChecks[OLDC.firstmap + i]
			local flags = V_TRANSLUCENT
			
			if clear or visited then flags = $ & ~V_TRANSLUCENT end
			if clear then flags = $ | V_GREENMAP end
			
			--v.drawCropped(startx - 10*FRACUNIT, starty, 8*FRACUNIT/128, 8*FRACUNIT/128, v.cachePatch("WARP-0"..j), 0, nil, 12*FRACUNIT, 12*FRACUNIT, 104*FRACUNIT, 104*FRACUNIT)
			v.drawString(startx, starty, GetMapName(OLDC.firstmap + i), flags|V_ALLOWLOWERCASE, "thin-fixed")
		end
		
		if topitem < maxitems
			v.drawString(startx - 12*FRACUNIT, starty - 2*FRACUNIT - arrowpos, "\027", V_YELLOWMAP, "fixed")
		end
	else
		local reversestartx = pivotx + sizex*FRACUNIT/2 - 8*FRACUNIT
		
		for _, i in ipairs(mplevelslist)
			starty = $ + 10*FRACUNIT
			
			local flags = not multiplayer and V_TRANSLUCENT or 0
			
			local color
			
			if i[2] == "Match" then color = V_REDMAP end
			if i[2] == "Capture the Flag" then color = V_BLUEMAP end
			if i[2] == "Circuit" then color = V_GREENMAP end
			
			v.drawString(startx, starty, GetMapName(G_GetMapNumber(i[1])), flags|V_ALLOWLOWERCASE, "thin-fixed")
			v.drawString(reversestartx, starty, i[2], color|flags|V_ALLOWLOWERCASE, "thin-fixed-right")
		end
		
		if not multiplayer
			v.drawString(pivotx, starty + 30*FRACUNIT, "Enter split-screen or online mode", V_ALLOWLOWERCASE, "thin-fixed-center")
			v.drawString(pivotx, starty + 38*FRACUNIT, "to access these levels.", V_ALLOWLOWERCASE, "thin-fixed-center")
		end
	end
	
	local offs = 8*FRACUNIT
	local button = v.cachePatch("BUTTON"..((leveltime/TICRATE) % 2) + 3)
	local buttonx = pivotx - sizex*FRACUNIT/2 + 8*FRACUNIT
	local buttony = pivoty + sizey*FRACUNIT/2 - 6*FRACUNIT
	
	v.drawScaled(buttonx, buttony, FRACUNIT/2, button)
	v.drawString(buttonx + offs, buttony - 3*FRACUNIT, "Close", V_ALLOWLOWERCASE, "thin-fixed")
	
	button = v.cachePatch("BUTTON"..((leveltime/TICRATE) % 2) + 1)
	buttonx = pivotx + sizex*FRACUNIT/2 - 8*FRACUNIT
	
	v.drawScaled(buttonx, buttony, FRACUNIT/2, button)
	v.drawString(buttonx - offs, buttony - 3*FRACUNIT, "Toggle view", V_ALLOWLOWERCASE, "thin-fixed-right")
	
	v.drawStretched(pivotx - xscale*sizex/2, pivoty - xscale*sizey/2, xscale*sizex/64, yscale*sizey/64, bg, V_90TRANS)
end)