freeslot("SPR_MMAR")

local cv_minimap = CV_RegisterVar({
	name = "oldc_minimap",
	defaultvalue = "Off",
	PossibleValue = CV_OnOff
})

local minimap

local mapcenterx, mapcentery = 160*FRACUNIT, 100*FRACUNIT
local mapscalex, mapscaley = FixedDiv(23456, 1337), FixedDiv(19744, 1128)
local hudscale = 5
local startx, starty = 0, 0

local thingstotrack = {}

addHook("PreThinkFrame", do thingstotrack = {} end)

for _, t in ipairs({MT_LEVELSPOT, MT_BANANAPORTER, MT_CHARASWAPPER, MT_OLDC_INFOSCREEN})
	print(cv_minimap.value)
	
	addHook("MobjThinker", function(mo)
		table.insert(thingstotrack, mo)
	end, t)
end

local shutup = 0

addHook("PlayerThink", function(p)
	if gamemap != (OLDC and OLDC.hub) then return end
	if shutup > 3*TICRATE then return end
	
	shutup = $ + 1
	
	if shutup == 3*TICRATE
		CONS_Printf(p, "\130Feeling lost? Turn on the minimap with \131oldc_minimap on\130!")
		S_StartSound(nil, sfx_menu1, p)
	end
end)

addHook("PlayerSpawn", function(p)
	if gamemap != (OLDC and OLDC.hub) then return end
	if not p.mo return end
	
	startx, starty = p.mo.y, p.mo.x
end)

-- The map is actually rotated 90 CCW so we're sapping the axes
local function getMinimapPosition(x, y)
	return FixedDiv((y - startx), mapscalex * hudscale), FixedDiv((x - starty), mapscaley * hudscale)
end

addHook("HUD", function(v, p)
	if gamemap != (OLDC and OLDC.hub) return end
	if not cv_minimap.value return end
	if OLDC and OLDC.Data.voteInfo.voting return true end
	if p.oldc_infoscreen_interacting return end
	if p.charaswap return end
	
	minimap = v.cachePatch("MINIMAP")
	
	local x, y = getMinimapPosition(p.mo.x, p.mo.y)
	
	v.drawScaled(mapcenterx - x, mapcentery - y, FRACUNIT/hudscale, minimap, V_70TRANS)
	
	for k, t in ipairs(thingstotrack)
		if not t.valid continue end
		
		local dotx, doty = getMinimapPosition(t.x, t.y)
		
		if t.type == MT_LEVELSPOT
			v.drawScaled(mapcenterx - x + dotx, mapcentery - y + doty, FRACUNIT/3, v.getSpritePatch("MAPP"), V_30TRANS)
		elseif t.type == MT_BANANAPORTER and t.triggerer
			local patch = v.cachePatch("TTL0"..(t.portaltag - 999))
			v.drawScaled(mapcenterx - x + dotx - patch.width*FRACUNIT/12, mapcentery - y + doty - patch.height*FRACUNIT/12, FRACUNIT/6, patch, V_30TRANS)
		elseif t.type == MT_CHARASWAPPER
			--local patch = v.getSprite2Patch("sonic", SPR2_LIFE)
			--v.drawScaled(mapcenterx - x + dotx, mapcentery - y + doty, FRACUNIT/3, patch, V_30TRANS, v.getColormap("sonic", SKINCOLOR_BLUE))
			
			v.drawScaled(mapcenterx - x + dotx, mapcentery - y + doty, FRACUNIT/4, v.getSpritePatch(SPR_TVRC, C), V_30TRANS)
		elseif t.type == MT_OLDC_INFOSCREEN
			v.drawScaled(mapcenterx - x + dotx, mapcentery - y + doty, FRACUNIT/6, v.cachePatch("INFOICON"), V_30TRANS)
		end
	end
	
	v.drawScaled(mapcenterx, mapcentery, FRACUNIT/8, v.getSpritePatch(SPR_MMAR, 0, 0, p.mo.angle + ANGLE_180), 0, v.getColormap(p.mo.skin, p.mo.color))
end)